<!doctype html>
<html lang="en">
<head>
  <meta charset="UTF-8">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <title>Pizza Map</title>
  <link rel="stylesheet" href="../src/index.css">
</head>
<body>
  <div id="pizzaz-root"></div>
  <script type="module" src="../src/pizzaz/index.jsx"></script>
</body>
</html>
